
--[[
    This file is a library to setup a lua 5.1 restricted environement
    ----

    Copyright (C) 2008-2011  TsT <tst@worldmaster.fr>

    This program is free software: you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of
    the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.
]]--

--[[
  Hello,
  If you found a bug or have a question, don't hesitate to contact me
  by email at <tst@worldmaster.fr> (in french or english please)
  Thanks,
  Have fun!
]]--

--[[ VERSION=0.1.0 20110429 ]]--
local luajailversion = "0.1.0 20110429"

assert(_VERSION and _VERSION == "Lua 5.1",
	"Version mismatch. This script are made for Lua 5.1")

assert(setfenv, "setfenv require")

--[[ debug ]]--
--print("host: _G", _G)
--

local function jailprint(...) print("jail:", ...) end

local jail = {}
jail._VERSION = luajailversion
jail._G = jail

-- getfenv() or getfenv(0) return the out-of-jail _G
local function jailgetfenv(f)
	if type(f) == "function" then
		return getfenv(f)
	end
	return jail._G
end

jail._VERSION       = _VERSION		-- ok
jail.assert         = assert		-- ok
collectgarbage      = collectgarbage	-- seems ok
--dofile				-- dangerous : go outside the love filesystem jail
jail.error = error			-- seems ok
jail.getfenv        = jailgetfenv	-- RISK: lua env jailbreak risk
jail.getmetatable   = getmetatable	-- ok
jail.ipairs         = ipairs		-- ok
jail.load           = load		-- seems ok
--loadfile				-- dangerous : go outside the love filesystem jail
jail.loadstring     = loadstring	-- seems ok
jail.module         = module		-- remove it ?
jail.next           = next		-- ok
jail.pairs          = pairs		-- ok
jail.pcall          = pcall		-- ok
jail.print          = jailprint		-- ok (for debug)
jail.rawequal       = rawequal		-- ok
jail.rawget         = rawget		-- ok
jail.rawset         = rawset		-- ok
jail.require        = require		-- TODO
jail.select         = select		-- ok
--setfenv       = setfenv		-- seems not ok, is able to reset the current env ?
jail.setmetatable   = setmetatable	-- ok
jail.tonumber       = tonumber		-- ok
jail.tostring       = tostring		-- ok
jail.type           = type		-- ok
jail.unpack         = unpack		-- ok
jail.xpcall         = xpcall		-- ok


jail.os = {}
jail.os.clock     = os.clock		-- ok
jail.os.date      = os.date		-- ok
jail.os.difftime  = os.difftime		-- ok
jail.os.getenv    = os.getenv		-- ok
jail.os.setlocale = os.setlocale	-- seems ok
jail.os.time      = os.time		-- ok
--jail.os.execute   = os.execute	-- RISK
--jail.os.exit      = os.exit		-- ?
--jail.os.remove    = os.remove		-- RISK
--jail.os.rename    = os.rename		-- RISK
--jail.os.tmpname   = os.tmpname	-- RISK

jail.io = {}
jail.io.flush        = io.flush		-- seems ok
-- RISK: file loading jailbreak risk	-- patched, ok
jail.io.input = function() -- no file argument
        return io.input()
end

-- RISK: file writing jailbreak risk	-- patched, ok
jail.io.output       = function() -- no file argument
        return io.output()
end
jail.io.read         = io.read -- really ?
jail.io.stderr       = io.stderr	-- ok
jail.io.stdin        = io.stdin		-- ok
jail.io.stdout       = io.stdout	-- ok
jail.io.type         = io.type		-- ok
jail.io.write        = io.write		-- seems ok (is for stdout, not file)
--jail.io.open         = io.open	-- denied by love policy
--jail.io.close        = io.close	-- denied by love policy
--jail.io.lines        = io.lines	-- denied by love policy
--jail.io.popen        = io.popen	-- RISK
--jail.io.tmpfile      = io.tmpfile	-- denied by love policy

jail.math   = math			-- ok
jail.string = string			-- ok
jail.table  = table			-- ok

-- the command line arguments
jail.arg    = arg			-- ok


--[[ TO SEE :
coroutine				-- seems not ok
package					-- TODO
debug					-- seems not ok
file					-- denied by love policy
]]--

-- namespace in global env
luajail = {}
luajail.jail = jail
luajail.disable = false
--function luajail.setupJailFor(f) setfenv(f, jail) end
function luajail.run(f, ...)
	if not luajail.disable then
		setfenv(f, jail)
	end
	return f(...)
end

